/*
   Copyright 2014-2019 Zephyr Software, LLC.

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/

#include <string>
#include <unistd.h>
#include <iostream>
#include <sstream>
#include <set>

namespace Zipr_SDK
{
	using namespace std;

	class ZiprOption_t 
	{
		protected:
			ZiprOption_t() { } 
			ZiprOption_t(const ZiprOption_t& copy) = delete;
		public:
			virtual ~ZiprOption_t() { } 
 
			virtual string getValueString()  const = 0; 
			virtual string getNamespace()    const = 0;
			virtual string getKey()          const = 0;
			virtual string getDescription()  const = 0;
			virtual bool areRequirementMet() const = 0;

			virtual void setRequired(bool req=true) = 0;

	};

	template <class T>
	class ZiprTypedOption_t : virtual public ZiprOption_t 
	{
		protected:
			ZiprTypedOption_t() { } 
			ZiprTypedOption_t(const ZiprTypedOption_t& copy) = delete;
		public:
			virtual ~ZiprTypedOption_t() { } 

			// Cast the option to it's type (e.g., string or bool)
                        virtual T getValue() const  = 0;
                        virtual operator T() const  = 0;

	};

	class ZiprStringOption_t : virtual public ZiprTypedOption_t<string>
	{
		protected:
			ZiprStringOption_t() { } 
			ZiprStringOption_t(const ZiprStringOption_t& copy) = delete;
		public:
			virtual ~ZiprStringOption_t() { } 
			virtual void setValue(const string& ) = 0; 
                        
	};

	class ZiprBooleanOption_t : virtual public ZiprTypedOption_t<bool>
	{
		protected:
			ZiprBooleanOption_t() { } 
			ZiprBooleanOption_t(const ZiprBooleanOption_t& copy) = delete;
		public:
			virtual ~ZiprBooleanOption_t() { } 
			virtual void setValue(const bool& ) = 0; 
                        
	};

	class ZiprIntegerOption_t : virtual public ZiprTypedOption_t<size_t>
	{
		protected:
			ZiprIntegerOption_t() { } 
			ZiprIntegerOption_t(const ZiprIntegerOption_t& copy) = delete;
		public:
			virtual ~ZiprIntegerOption_t() { } 
			virtual void setValue(const size_t& ) = 0; 
                        
	};

	class ZiprDoubleOption_t : public ZiprTypedOption_t<double>
	{
		protected:
			ZiprDoubleOption_t() { } 
			ZiprDoubleOption_t(const ZiprDoubleOption_t& copy) = delete;
		public:
			virtual ~ZiprDoubleOption_t() { } 
			virtual void setValue(const double& ) = 0; 
                        
	};

	class ZiprOptionsNamespace_t 
	{
		protected:
			ZiprOptionsNamespace_t() { } 
			ZiprOptionsNamespace_t(const ZiprOptionsNamespace_t& copy) = delete;
		public:
			virtual ~ZiprOptionsNamespace_t() { } 
			virtual string getNamespaceString() const = 0 ;
			virtual ZiprStringOption_t*  getStringOption (const string& name, const string &description="", const string& default_value="")    = 0; 
			virtual ZiprIntegerOption_t* getIntegerOption(const string& name, const string &description="", const size_t& default_value=0)     = 0; 
			virtual ZiprBooleanOption_t* getBooleanOption(const string& name, const string &description="", const bool  & default_value=false) = 0; 
			virtual ZiprDoubleOption_t*  getDoubleOption (const string& name, const string &description="", const double& default_value=0.0)   = 0; 
	};

	class ZiprOptionsManager_t 
	{
		protected:
			ZiprOptionsManager_t() { } 
			ZiprOptionsManager_t(const ZiprOptionsManager_t& copy) = delete;

		public:
			virtual ~ZiprOptionsManager_t() { } 

                        virtual bool areRequirementsMet()                          const = 0;
			virtual ZiprOptionsNamespace_t* getNamespace(const string& name) = 0;


	};
}
