/*
   Copyright 2014-2019 Zephyr Software, LLC.

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/

namespace Zipr_SDK
{
	using namespace std;
	using namespace IRDB_SDK;

	using InstructionLocationMap_t = map<Instruction_t*,RangeAddress_t>;

	using ZiprPreference_t = enum ZiprPreference 
		{
			None,
			Must,	
		};

	class ZiprPluginInterface_t
	{
		public:
			virtual void doPinningBegin() { }
			virtual void doPinningEnd() { }
			virtual void doDollopBegin() { }
			virtual void doDollopEnd() { }
			virtual void doCallbackLinkingBegin() { }
			virtual void doCallbackLinkingEnd() { }

			virtual ZiprPreference_t addressDollop(const Dollop_t *dollop, const RangeAddress_t &source, Range_t &place, bool &coalesce, bool &fallthrough_ok)
				{ return None; } 
			
			virtual bool willPluginPlop(Instruction_t*) 
				{ return false; }

			virtual size_t getInsnSize(Instruction_t*, bool) 
				{ return 0; }

			/*
			 * DollopEntryOpeningSize()
			 * DollopEntryClosingSize()
			 *
			 * A plugin that wants to put padding into a
			 * dollop entry before/after an instruction 
			 * should return the size of that padding in 
			 * these functions.
			 */
			virtual size_t getDollopEntryOpeningSize(DollopEntry_t*) { return 0; }
			virtual size_t getDollopEntryClosingSize(DollopEntry_t*) { return 0; }

			/*
			 * PlopDollopEntry()
			 *
			 * This function is invoked by Zipr for a plugin to 
			 * plop a dollop entry. The entry to plop is given as
			 * the first parameter. The plugin writer can use
			 * it's Place() function to determine the beginning
			 * of the space allocated for this dollop entry. 
			 *
			 * entry: A pointer to the dollop entry to be plopped.
			 *
			 * instruction_address: Because multiple plugins 
			 * may want to plop the same dollop entry, the current 
			 * address of the instruction within the entry is 
			 * passed in this parameter. If the plugin moves that 
			 * instruction, it must update the parameter.
			 *
			 * instruction_size: The size of the instruction itself.
			 *
			 * placed_instruction: A flag that the plugin sets
			 * to tell Zipr whether this plugin placed the
			 * instruction or whether it just wrote into the entry
			 * surrounding the instruction. The plugin should set
			 * this value appropriately. The default is false.
			 *
			 * zipr: Pointer to a Zipr object for plugin access to
			 * functionality provided by zipr.
			 *
			 * As output, the implementation must provide the 
			 * address of the memory that "concludes" the 
			 * dollop entry.
			 */
			virtual RangeAddress_t plopDollopEntry(DollopEntry_t *entry,
				RangeAddress_t &instruction_address,
				RangeAddress_t &target_address,
				size_t instruction_size,
				bool &placed_instruction) 
				{ return 0; }

			virtual string toString() 
				{ return "NamelessPlugin"; }

			virtual ZiprPreference_t retargetPin(const RangeAddress_t &, const Dollop_t *, RangeAddress_t &)
				{ return None; }

			virtual ZiprPreference_t retargetCallback(const RangeAddress_t &, const DollopEntry_t *, RangeAddress_t &)
				{ return None; }

			virtual RangeAddress_t doPlaceScoopsBegin(const RangeAddress_t max_addr)
				{ return max_addr; }

			virtual RangeAddress_t doPlaceScoopsEnd(const RangeAddress_t max_addr)
				{ return max_addr; }

	};

}
